@@ -1,4 +1,4 @@
-/* $Id: minissdpd.c,v 1.22 2011/07/29 08:44:04 nanard Exp $ */
+/* $Id: minissdpd.c,v 1.26 2011/10/07 09:45:19 nanard Exp $ */
 /* MiniUPnP project
  * (c) 2007-2011 Thomas Bernard
  * website : http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
@@ -34,6 +34,7 @@
 #include "openssdpsocket.h"
 #include "daemonize.h"
 #include "codelength.h"
+#include "ifacewatch.h"
 
 /* current request management stucture */
 struct reqelem {
@@ -74,7 +75,8 @@ unsigned int upnp_configid = 1337;
  *  -1 : error
  *   0 : the device was updated
  *   1 : the device was new    */
-int updateDevice(const struct header * headers, time_t t)
+static int
+updateDevice(const struct header * headers, time_t t)
 {
 	struct device ** pp = &devlist;
 	struct device * p = *pp;	/* = devlist; */
@@ -136,7 +138,8 @@ int updateDevice(const struct header * headers, time_t t)
  * return value :
  *    0 : no device removed
  *   -1 : device removed */
-int removeDevice(const struct header * headers)
+static int
+removeDevice(const struct header * headers)
 {
 	struct device ** pp = &devlist;
 	struct device * p = *pp;	/* = devlist */
@@ -218,8 +221,9 @@ struct service {
 LIST_HEAD(servicehead, service) servicelisthead;
 
 /* Process M-SEARCH requests */
-void processMSEARCH(int s, const char * st, int st_len,
-                    const struct sockaddr * addr)
+static void
+processMSEARCH(int s, const char * st, int st_len,
+               const struct sockaddr * addr)
 {
 	struct service * serv;
 #ifdef ENABLE_IPV6
@@ -274,6 +278,22 @@ void processMSEARCH(int s, const char * st, int st_len,
 	}
 }
 
+/**
+ * helper function.
+ * reject any non ASCII or non printable character.
+ */
+static int
+containsForbiddenChars(const unsigned char * p, int len)
+{
+	while(len > 0) {
+		if(*p < ' ' || *p >= '\x7f')
+			return 1;
+		p++;
+		len--;
+	}
+	return 0;
+}
+
 #define METHOD_MSEARCH 1
 #define METHOD_NOTIFY 2
 
@@ -284,8 +304,9 @@ void processMSEARCH(int s, const char * st, int st_len,
  *    -1 : a device was removed
  *     0 : no device removed nor added
  *     1 : a device was added.  */
-int ParseSSDPPacket(int s, const char * p, ssize_t n,
-                    const struct sockaddr * addr)
+static int
+ParseSSDPPacket(int s, const char * p, ssize_t n,
+                const struct sockaddr * addr)
 {
 	const char * linestart;
 	const char * lineend;
@@ -436,17 +457,26 @@ int ParseSSDPPacket(int s, const char * p, ssize_t n,
 /* OpenUnixSocket()
  * open the unix socket and call bind() and listen()
  * return -1 in case of error */
-int
+static int
 OpenUnixSocket(const char * path)
 {
 	struct sockaddr_un addr;
 	int s;
+	int rv;
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
 	if(s < 0)
 	{
 		syslog(LOG_ERR, "socket(AF_UNIX): %m");
 		return -1;
 	}
+	/* unlink the socket pseudo file before binding */
+	rv = unlink(path);
+	if(rv < 0 && errno != ENOENT)
+	{
+		syslog(LOG_ERR, "unlink(unixsocket, \"%s\"): %m", path);
+		close(s);
+		return -1;
+	}
 	addr.sun_family = AF_UNIX;
 	strncpy(addr.sun_path, path, sizeof(addr.sun_path));
 	if(bind(s, (struct sockaddr *)&addr,
@@ -499,11 +529,15 @@ void processRequest(struct reqelem * req)
 	t = time(NULL);
 	type = buf[0];
 	p = buf + 1;
-	DECODELENGTH(l, p);
+	DECODELENGTH_CHECKLIMIT(l, p, buf + n);
 	if(p+l > buf+n) {
 		syslog(LOG_WARNING, "bad request (length encoding)");
 		goto error;
 	}
+	if(l == 0) {
+		syslog(LOG_WARNING, "bad request (length=0)");
+		goto error;
+	}
 	syslog(LOG_INFO, "(s=%d) request type=%d str='%.*s'",
 	       req->socket, type, l, p);
 	switch(type) {
@@ -584,6 +618,10 @@ void processRequest(struct reqelem * req)
 			syslog(LOG_ERR, "cannot allocate memory");
 			goto error;
 		}
+		if(containsForbiddenChars(p, l)) {
+			syslog(LOG_ERR, "bad request (st contains forbidden chars)");
+			goto error;
+		}
 		newserv->st = malloc(l + 1);
 		if(!newserv->st) {
 			syslog(LOG_ERR, "cannot allocate memory");
@@ -596,11 +634,15 @@ void processRequest(struct reqelem * req)
 			syslog(LOG_WARNING, "bad request (missing usn)");
 			goto error;
 		}
-		DECODELENGTH(l, p);
+		DECODELENGTH_CHECKLIMIT(l, p, buf + n);
 		if(p+l > buf+n) {
 			syslog(LOG_WARNING, "bad request (length encoding)");
 			goto error;
 		}
+		if(containsForbiddenChars(p, l)) {
+			syslog(LOG_ERR, "bad request (usn contains forbidden chars)");
+			goto error;
+		}
 		syslog(LOG_INFO, "usn='%.*s'", l, p);
 		newserv->usn = malloc(l + 1);
 		if(!newserv->usn) {
@@ -610,7 +652,15 @@ void processRequest(struct reqelem * req)
 		memcpy(newserv->usn, p, l);
 		newserv->usn[l] = '\0';
 		p += l;
-		DECODELENGTH(l, p);
+		DECODELENGTH_CHECKLIMIT(l, p, buf + n);
+		if(p+l > buf+n) {
+			syslog(LOG_WARNING, "bad request (length encoding)");
+			goto error;
+		}
+		if(containsForbiddenChars(p, l)) {
+			syslog(LOG_ERR, "bad request (server contains forbidden chars)");
+			goto error;
+		}
 		syslog(LOG_INFO, "server='%.*s'", l, p);
 		newserv->server = malloc(l + 1);
 		if(!newserv->server) {
@@ -620,7 +670,15 @@ void processRequest(struct reqelem * req)
 		memcpy(newserv->server, p, l);
 		newserv->server[l] = '\0';
 		p += l;
-		DECODELENGTH(l, p);
+		DECODELENGTH_CHECKLIMIT(l, p, buf + n);
+		if(p+l > buf+n) {
+			syslog(LOG_WARNING, "bad request (length encoding)");
+			goto error;
+		}
+		if(containsForbiddenChars(p, l)) {
+			syslog(LOG_ERR, "bad request (location contains forbidden chars)");
+			goto error;
+		}
 		syslog(LOG_INFO, "location='%.*s'", l, p);
 		newserv->location = malloc(l + 1);
 		if(!newserv->location) {
@@ -699,8 +757,11 @@ int main(int argc, char * * argv)
 	int s_ssdp = -1;	/* udp socket receiving ssdp packets */
 #ifdef ENABLE_IPV6
 	int s_ssdp6 = -1;	/* udp socket receiving ssdp packets IPv6*/
+#else
+#define s_ssdp6 (-1)
 #endif
 	int s_unix = -1;	/* unix socket communicating with clients */
+	int s_ifacewatch = -1;	/* socket to receive Route / network interface config changes */
 	int s;
 	LIST_HEAD(reqstructhead, reqelem) reqlisthead;
 	struct reqelem * req;
@@ -758,18 +819,6 @@ int main(int argc, char * * argv)
 		return 1;
 	}
 
-	/* daemonize or in any case get pid ! */
-	if(debug_flag)
-		pid = getpid();
-	else {
-#ifdef USE_DAEMON
-		if(daemon(0, 0) < 0)
-			perror("daemon()");
-		pid = getpid();
-#else
-		pid = daemonize();
-#endif
-	}
 	/* open log */
 	openlog("minissdpd",
 	        LOG_CONS|LOG_PID|(debug_flag?LOG_PERROR:0),
@@ -783,8 +832,6 @@ int main(int argc, char * * argv)
 		return 1;
 	}
 
-	writepidfile(pidfilename, pid);
-
 	/* set signal handlers */
 	memset(&sa, 0, sizeof(struct sigaction));
 	sa.sa_handler = sigterm;
@@ -800,6 +847,8 @@ int main(int argc, char * * argv)
 		ret = 1;
 		goto quit;
 	}
+	/* open route/interface config changes socket */
+	s_ifacewatch = OpenAndConfInterfaceWatchSocket();
 	/* open UDP socket(s) for receiving SSDP packets */
 	s_ssdp = OpenAndConfSSDPReceiveSocket(n_if_addr, if_addr, 0);
 	if(s_ssdp < 0)
@@ -860,17 +909,37 @@ int main(int argc, char * * argv)
 	}
 #endif
 
+	/* daemonize or in any case get pid ! */
+	if(debug_flag)
+		pid = getpid();
+	else {
+#ifdef USE_DAEMON
+		if(daemon(0, 0) < 0)
+			perror("daemon()");
+		pid = getpid();
+#else
+		pid = daemonize();
+#endif
+	}
+
+	writepidfile(pidfilename, pid);
+
 	/* Main loop */
 	while(!quitting)
 	{
 		/* fill readfds fd_set */
 		FD_ZERO(&readfds);
-		FD_SET(s_ssdp, &readfds);
+		if(s_ssdp >= 0) {
+			FD_SET(s_ssdp, &readfds);
+		}
 #ifdef ENABLE_IPV6
 		if(s_ssdp6 >= 0) {
 			FD_SET(s_ssdp6, &readfds);
 		}
 #endif
+		if(s_ifacewatch >= 0) {
+			FD_SET(s_ifacewatch, &readfds);
+		}
 		FD_SET(s_unix, &readfds);
 		for(req = reqlisthead.lh_first; req; req = req->entries.le_next)
 		{
@@ -986,6 +1055,10 @@ int main(int argc, char * * argv)
 				}
 			}
 		}
+		/* processing route/network interface config changes */
+		if((s_ifacewatch >= 0) && FD_ISSET(s_ifacewatch, &readfds)) {
+			ProcessInterfaceWatch(s_ifacewatch, s_ssdp, s_ssdp6, n_if_addr, if_addr);
+		}
 	}
 
 	/* closing and cleaning everything */
@@ -1003,9 +1076,13 @@ quit:
 	if(s_unix >= 0) {
 		close(s_unix);
 		s_unix = -1;
+		if(unlink(sockpath) < 0)
+			syslog(LOG_ERR, "unlink(%s): %m", sockpath);
+	}
+	if(s_ifacewatch >= 0) {
+		close(s_ifacewatch);
+		s_ifacewatch = -1;
 	}
-	if(unlink(sockpath) < 0)
-		syslog(LOG_ERR, "unlink(%s): %m", sockpath);
 	if(unlink(pidfilename) < 0)
 		syslog(LOG_ERR, "unlink(%s): %m", pidfilename);
 	closelog();
