@@ -1,4 +1,4 @@
-/* $Id: openssdpsocket.c,v 1.5 2011/06/30 20:58:40 nanard Exp $ */
+/* $Id: openssdpsocket.c,v 1.8 2011/10/07 09:21:03 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2011 Thomas Bernard
@@ -73,8 +73,8 @@ GetIfAddrIPv4(const char * ifaddr)
  * @param ifaddr	the IPv4 address or interface name
  * @param ipv6	IPv6 or IPv4
  * return -1 on error, 0 on success */
-static int
-AddMulticastMembership(int s, const char * ifaddr, int ipv6)
+int
+AddDropMulticastMembership(int s, const char * ifaddr, int ipv6, int drop)
 {
 	struct ip_mreq imr;	/* Ip multicast membership */
 #ifdef ENABLE_IPV6
@@ -89,18 +89,20 @@ AddMulticastMembership(int s, const char * ifaddr, int ipv6)
 		memset(&mr, 0, sizeof(mr));
 		inet_pton(AF_INET6, LL_SSDP_MCAST_ADDR, &mr.ipv6mr_multiaddr);
 		mr.ipv6mr_interface = ifindex;
-		if(setsockopt(s, IPPROTO_IPV6, IPV6_JOIN_GROUP,
+		if(setsockopt(s, IPPROTO_IPV6, drop ? IPV6_LEAVE_GROUP : IPV6_JOIN_GROUP,
 		   &mr, sizeof(struct ipv6_mreq)) < 0)
 		{
-			syslog(LOG_ERR, "setsockopt(udp, IPV6_JOIN_GROUP)(%s): %m",
+			syslog(LOG_ERR, "setsockopt(udp, %s)(%s): %m",
+			       drop ? "IPV6_LEAVE_GROUP" : "IPV6_JOIN_GROUP",
 			       ifaddr);
 			return -1;
 		}
 		inet_pton(AF_INET6, SL_SSDP_MCAST_ADDR, &mr.ipv6mr_multiaddr);
-		if(setsockopt(s, IPPROTO_IPV6, IPV6_JOIN_GROUP,
+		if(setsockopt(s, IPPROTO_IPV6, drop ? IPV6_LEAVE_GROUP : IPV6_JOIN_GROUP,
 		   &mr, sizeof(struct ipv6_mreq)) < 0)
 		{
-			syslog(LOG_ERR, "setsockopt(udp, IPV6_JOIN_GROUP)(%s): %m",
+			syslog(LOG_ERR, "setsockopt(udp, %s)(%s): %m",
+			       drop ? "IPV6_LEAVE_GROUP" : "IPV6_JOIN_GROUP",
 			       ifaddr);
 			return -1;
 		}
@@ -120,10 +122,11 @@ AddMulticastMembership(int s, const char * ifaddr, int ipv6)
 			return -1;
 		}
 		
-		if (setsockopt(s, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+		if (setsockopt(s, IPPROTO_IP, drop ? IP_DROP_MEMBERSHIP : IP_ADD_MEMBERSHIP,
 		    (void *)&imr, sizeof(struct ip_mreq)) < 0)
 		{
-	        syslog(LOG_ERR, "setsockopt(udp, IP_ADD_MEMBERSHIP)(%s): %m",
+	        syslog(LOG_ERR, "setsockopt(udp, %s)(%s): %m",
+			       drop ? "IP_DROP_MEMBERSHIP" : "IP_ADD_MEMBERSHIP",
 			       ifaddr);
 			return -1;
 	    }
@@ -140,8 +143,8 @@ OpenAndConfSSDPReceiveSocket(int n_listen_addr,
                              int ipv6)
 {
 	int s;
+	int opt = 1;
 #ifdef ENABLE_IPV6
-	int on = 1;
 	struct sockaddr_storage sockname;
 #else
 	struct sockaddr_in sockname;
@@ -164,7 +167,7 @@ OpenAndConfSSDPReceiveSocket(int n_listen_addr,
 	{
 #ifdef IPV6_V6ONLY
 		if(setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY,
-		              (char *)&on, sizeof(on)) < 0)
+		              (char *)&opt, sizeof(opt)) < 0)
 		{
 			syslog(LOG_WARNING, "setsockopt(IPV6_V6ONLY): %m");
 		}
@@ -194,6 +197,11 @@ OpenAndConfSSDPReceiveSocket(int n_listen_addr,
 	sockname_len = sizeof(struct sockaddr_in);
 #endif
 
+	if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)) < 0)
+	{
+		syslog(LOG_WARNING, "setsockopt(SO_REUSEADDR): %m");
+	}
+
     if(bind(s, (struct sockaddr *)&sockname, sockname_len) < 0)
 	{
 		syslog(LOG_ERR, "bind(udp%s): %m", ipv6 ? "6" : "");
@@ -204,12 +212,11 @@ OpenAndConfSSDPReceiveSocket(int n_listen_addr,
 	while(n_listen_addr>0)
 	{
 		n_listen_addr--;
-		if(AddMulticastMembership(s, listen_addr[n_listen_addr], ipv6) < 0)
+		if(AddDropMulticastMembership(s, listen_addr[n_listen_addr], ipv6, 0) < 0)
 		{
-			syslog(LOG_WARNING, "Failed to add membership for interface %s. EXITING", 
+			syslog(LOG_WARNING, "Failed to add IPv%d multicast membership for interface %s.",
+			       ipv6 ? 6 : 4,
 			       listen_addr[n_listen_addr] );
-			close(s);
-			return -1;
 		}
 	}
 
