@@ -1,4 +1,4 @@
-/* $Id: openssdpsocket.h,v 1.3 2011/05/23 12:23:21 nanard Exp $ */
+/* $Id: openssdpsocket.h,v 1.4 2011/07/29 15:21:13 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2011 Thomas Bernard
@@ -16,9 +16,19 @@
  * @return socket
  */
 int
-OpenAndConfSSDPReceiveSocket(int n_listen_addr,
-							 const char * * listen_addr,
+OpenAndConfSSDPReceiveSocket(int n_if_addr,
+                             const char * * if_addr,
                              int ipv6);
 
+/**
+ * Add or Drop the multicast membership for SSDP on the interface
+ * @param s	the socket
+ * @param ifaddr	the IPv4 address or interface name
+ * @param ipv6	IPv6 or IPv4
+ * @param drop	0 to add, 1 to drop
+ * return -1 on error, 0 on success */
+int
+AddDropMulticastMembership(int s, const char * ifaddr, int ipv6, int drop);
+
 #endif
 
